<?php

namespace Drupal\eca_danse\Event;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Plugin\DataType\EntityAdapter;
use Drupal\eca\Plugin\DataType\DataTransferObject;
use Drupal\eca\Token\DataProviderInterface;

/**
 * Provides the event that should select recipients for DANSE.
 *
 * @internal
 *   This class is not meant to be used as a public API. It is subject for name
 *   change or may be removed completely, also on minor version updates.
 *
 * @package Drupal\danse\Event
 */
class RecipientSelection extends BaseEvent implements DataProviderInterface {

  protected ?DataTransferObject $recipients = NULL;

  public function init(): void {
    if ($this->recipients === NULL) {
      $this->recipients = DataTransferObject::create();
      $this->recipients->setValue([]);
    }
  }

  public function getRecipients(): array {
    $this->init();
    $result = [];
    foreach ($this->recipients as $item) {
      if (is_scalar($item)) {
        $result[] = $item;
      }
      elseif ($item instanceof ContentEntityInterface) {
        $result[] = $item->id();
      }
      elseif ($item instanceof EntityAdapter && $entity = $item->getValue()) {
        $result[] = $entity->id();
      }
    }
    return $result;
  }

  public function getData(string $key): ?DataTransferObject {
    if ($key === 'danse_recipients') {
      $this->init();
      return $this->recipients;
    }
    return NULL;
  }

  public function hasData(string $key): bool {
    return ($key === 'danse_recipients');
  }

}
