<?php

namespace Drupal\danse\Commands;

use Drupal\danse\Service;
use Drush\Commands\DrushCommands;

/**
 * A Drush commandfile.
 */
class DanseCommands extends DrushCommands {

  /**
   * The DANSE service.
   *
   * @var \Drupal\danse\Service
   */
  protected Service $danse;

  /**
   * DanseCommands constructor.
   *
   * @param \Drupal\danse\Service $danse
   *   The DANSE service.
   */
  public function __construct(Service $danse) {
    parent::__construct();
    $this->danse = $danse;
  }

  /**
   * Create notifications.
   *
   * @usage danse:notifications:create
   *   Creates all outstanding notifications.
   *
   * @command danse:notifications:create
   * @aliases dnc
   */
  public function createNotifications(): void {
    $this->danse->createNotifications();
  }

  /**
   * Get status of DANSE event tracking.
   *
   * @command danse:event-tracking:status
   */
  public function status(): void {
    if ($this->danse->isPaused()) {
      $this->io()->warning('DANSE event tracking is paused.');
    }
    else {
      $this->io()->success('DANSE event tracking is active.');
    }
  }

  /**
   * Pause DANSE event tracking.
   *
   * @command danse:event-tracking:pause
   */
  public function pause(): void {
    $this->danse->pause();
    $this->io()->success('DANSE event tracking paused.');
  }

  /**
   * Resume DANSE event tracking.
   *
   * @command danse:event-tracking:resume
   */
  public function resume(): void {
    $this->danse->resume();
    $this->io()->success('DANSE event tracking resumed.');
  }

}
