<?php

namespace Drupal\danse\Entity;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\DrupalKernel;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\danse\PluginManager;
use Drupal\danse\Query;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * View builder handler for DANSE events.
 */
final class EventView extends EntityViewBuilder {

  /**
   * The kernel.
   *
   * @var \Drupal\Core\DrupalKernel
   */
  protected DrupalKernel $kernel;

  /**
   * The request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $request;

  /**
   * The DANSE query service.
   *
   * @var \Drupal\danse\Query
   */
  protected Query $query;

  /**
   * The DANSE plugin manager.
   *
   * @var \Drupal\danse\PluginManager
   */
  protected PluginManager $pluginManager;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type): EventView {
    $instance = parent::createInstance($container, $entity_type);
    $instance->kernel = $container->get('kernel');
    $instance->request = $container->get('request_stack')->getCurrentRequest();
    $instance->query = $container->get('danse.query');
    $instance->pluginManager = $container->get('plugin.manager.danse.plugin');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $build) {
    $session = $this->request->getSession();
    /**
     * @var \Drupal\danse\Entity\EventInterface $event
     */
    $event = $build['#danse_event'];
    foreach ($this->query->findEventNotificationsForCurrentUser($event) as $notification) {
      try {
        $notification
          ->markSeen()
          ->save();
      }
      catch (EntityStorageException $e) {
        // @todo Log this exception.
      }
    }

    try {
      $plugin = $this->pluginManager->createInstance($event->getPluginId());
    }
    catch (PluginException $e) {
      // @todo Log this exception.
      return [];
    }
    $response = new RedirectResponse($plugin->getRedirectUrl($event)->toString());
    $session->save();
    $response->prepare($this->request);
    $this->kernel->terminate($this->request, $response);
    $response->send();
    exit;
  }

}
